open Graphics
open World

let state = ref None

let open_display world res =
  let h = Array.length world.map in
  let w = Array.length world.map.(0) in
  open_graph (Printf.sprintf " %dx%d" (w * res) ((h + 1) * res)) ;
  auto_synchronize false ;
  state := Some (res, "MARC POLO'S POOR CAML COMPALOR")

let attendre_une_touche () =
  match !state with
  | None -> failwith "GRAPHICAL DISPLAY WAS NOT OPEN, WTF DUDE"
  | Some _ -> wait_next_event [ Button_down ] |> ignore

let close_display () =
  match !state with
  | None -> failwith "GRAPHICAL DISPLAY WAS NOT OPEN, WTF DUDE"
  | Some _ -> close_graph () ; state := None

let set_text s =
  match !state with
  | None -> ()
  | Some (res, _) ->
     state := Some (res, (String.uppercase s))

let refresh world =
  match !state with
  | None -> failwith "GRAPHICAL DISPLAY WAS NOT OPEN, WTF DUDE"
  | Some (res, text) ->
     let h = Array.length world.map in
     let w = Array.length world.map.(0) in
     Sprites.background res w (h + 1) ;
     for y = 0 to h - 1 do
       for x = 0 to w - 1 do
	 match world.map.(h - y - 1).(x) with
	 | Lambda_man
         | Ghost
	 | Empty -> Sprites.empty (x * res) (y * res) res res
	 | Wall -> Sprites.wall (x * res) (y * res) res res
	 | Pill -> Sprites.pill (x * res) (y * res) res res
	 | Power_pill -> Sprites.power_pill (x * res) (y * res) res res
	 | Fruit ->
            match world.fruit with
            | None ->
               Sprites.empty (x * res) (y * res) res res
            | Some _ ->
               Sprites.fruit (x * res) (y * res) res res
            (* Sprites.lambda_man (x * res) (y * res) res res *)
	    (* Sprites.ghost (x * res) (y * res) res res *)
       done
     done ;
     Array.iter
       (fun g ->
        let h = Array.length world.map in
        let x = g.current_location.x in
        let y = h - g.current_location.y - 1 in
        Sprites.ghost (x * res) (y * res) res res)
       world.ghosts;
     let () =
       let h = Array.length world.map in
       let x = world.lambda_man.location.x in
       let y = h - world.lambda_man.location.y - 1 in
       Sprites.lambda_man (x * res) (y * res) res res
     in
     Sprites.draw_text text (res / 4) (h * res + res / 4) (res / 2) (res / 2) ;
     synchronize ()
