{
  open GccTypes
  let ios n =
    match n.[0] with
    | '0'..'9' -> I (int_of_string n)
    | _ -> L n
  let int= int_of_string

}

let ws = [' ' '\t']+
let endl = ws? (';' ([^ '\n']*))? '\n' (ws ?)
let lablname = ['a'-'z'] ( ['a'-'z''A'-'Z''0'-'9''_' '=' '@' '\'']* )
let num = ['0'-'9']+| lablname

      rule one_instruction = parse
        | "LDC" ws (num as n) { LDC (int n) }
        | "LD" ws (num as n) ws (num as n') { LD (int n, int n') }
        | "ADD" { ADD }
        | "SUB" { SUB }
        | "MUL" { MUL }
        | "DIV" { DIV }
        | "CEQ" { CEQ }
        | "CGT" { CGT }
        | "CGTE" { CGTE }
        | "ATOM" { ATOM }
        | "CONS" { CONS }
        | "CAR" { CAR }
        | "CDR" { CDR }
        | "SEL" ws (num as t) ws (num as f) { SEL (ios t,ios f) }
        | "JOIN" { JOIN }
        | "LDF" ws (num as f) { LDF (ios f) }
        | "AP" ws (num as n) { AP (int n) }
        | "RTN" { RTN }
        | "DUM" ws (num as s) { DUM (int s) }
        | "RAP" ws (num as n) { RAP (int n) }
        | "STOP" { STOP }
        | "TSEL" ws (num as t) ws (num as f) { TSEL (ios t,ios f) }
        | "TAP" ws (num as n) { TAP (int n) }
        | "TRAP" ws (num as n) { TRAP (int n) }
        | "ST" ws (num as n) ws (num as n') { ST (int n, int n') }
        | "DBUG" { DBUG }
        | (lablname as n) (ws?) ':' { LABEL (L n) }
        | endl { Lexing.new_line lexbuf; one_instruction lexbuf }
        | ws { one_instruction lexbuf }
        | eof { raise Exit }

            {
              let from_file f =
                let c = open_in f in
                let lexbuf = Lexing.from_channel c in
                let lines = ref [] in
                try
                  while true do
                    lines := one_instruction lexbuf :: !lines
                  done;
                  assert false
                with
                | Exit ->
                   (let res = Array.of_list (List.rev !lines) in
                    close_in c;
                    res)
                | e ->
                   (let p = Lexing.lexeme_start_p lexbuf in
                    Format.eprintf
                      "lex error file %s line %i char %i@."
                      f
                      p.Lexing.pos_lnum
                      (p.Lexing.pos_cnum - p.Lexing.pos_bol);
                    raise e)
            }
