type arg = I of int | L of string

type 'label ins =
  | LDC of int
  | LD of int * int
  | ADD
  | SUB
  | MUL
  | DIV
  | CEQ
  | CGT
  | CGTE
  | CONS
  | CAR
  | CDR
  | ATOM
  | SEL of 'label * 'label
  | JOIN
  | LDF of 'label
  | AP of int
  | RTN
  | DUM of int
  | RAP of int
  | STOP
  | TSEL of 'label * 'label
  | TAP of int
  | TRAP of int
  | ST of int * int
  | DBUG
  | BRK

  | LABEL of 'label
  | COMMENT of string

type 'a value =
  | TAG_INT of int
  | TAG_CONS of 'a value * 'a value
  | TAG_CLOSURE of 'a * 'a frame
  | TAG_DUM

and 'a frame = {
  parent : 'a frame option;
  values : 'a value array;
}

and 'a control =
  | TAG_JOIN of 'a
  | TAG_RET of 'a * 'a frame
  | TAG_STOP

type 'a program = 'a ins array

type 'a state = {
  data : 'a value list;
  control : 'a control list;
  env : 'a frame;
  pc : int;
  program : 'a ins array;
  mutable used : int;
}

type fault =
  | TAG_MISMATCH of string
  | CONTROL_MISMATCH of string
  | FRAME_MISMATCH of int * int
  | EMPTY_STACK of string
  | EMPTY_FRAME of string

type any_state =
  | Arg_state of arg state
  | Int_state of int state

exception MACHINE_STOP of any_state
exception BREAKPOINT of any_state

exception FAULT of fault


module StringMap = Map.Make(String)

