type cell =
  | Wall
  | Empty
  | Pill
  | Power_pill
  | Fruit
  | Lambda_man
  | Ghost

type row = cell list
type map = row list
type location = { x : int; y : int }

type direction = Up | Right | Down | Left

type lambda_man = {
  vitality : int; (* ticks before end of power pill activity *)
  location : location;
  direction : direction;
  lives : int;
  score : int;
}

type ghost_vitality =
  | Standard
  | Fright_mode
  | Invisible

type ghost =
    { vitality : ghost_vitality;
      location : location;
      direction : direction }
type fruit = int option

type world =
    { map : cell list list;
      lambda_man : lambda_man;
      ghosts : ghost list ;
      fruit : int option }

let rec retrieve_world value : world =
  let map = retrieve_map (car value) in
  let value = cdr value in
  let lambda_man = retrieve_lambda_man (car value) in
  let value = cdr value in
  let ghosts = retrieve_ghosts (car value) in
  let fruit = retrieve_fruit (cdr value) in
  { map ; lambda_man ; ghosts ; fruit }
and retrieve_map value =
  list_map retrieve_row (retrieve_list value)
and retrieve_row value =
  list_map retrieve_cell (retrieve_list value)  
and retrieve_cell v =
  match (magic v : int) with  
  | 0 -> Wall
  | 1 -> Empty
  | 2 -> Pill
  | 3 -> Power_pill
  | 4 -> Fruit
  | 5 -> Lambda_man
  | _ -> Ghost
and retrieve_fruit value =
  let v = (magic value : int) in
  if v = 0 then None else Some v
and retrieve_lambda_man value =
  let vitality : int = magic (car value) in
  let location = retrieve_location (cadr value) in
  let direction = retrieve_direction (caddr value) in
  let lives : int = magic (cadddr value) in
  let score : int = magic (cddddr value) in
  { vitality ; location ; direction ; lives ; score }
and retrieve_location value =
  { x = magic (car value) ; y = magic (cdr value) }
and retrieve_direction value =
  match (magic value : int) with
  | 0 -> Up
  | 1 -> Right
  | 2 -> Down
  | _ -> Left
and retrieve_ghosts value =
  list_map retrieve_ghost (retrieve_list value)
and retrieve_ghost value =
  let vitality = retrieve_ghost_vitality (car value) in
  let location = retrieve_location (cadr value) in
  let direction = retrieve_direction (cddr value) in
  { vitality ; location ; direction }
and retrieve_ghost_vitality value =
  match (magic value : int) with
  | 0 -> Standard
  | 1 -> Fright_mode
  | _ -> Invisible
and retrieve_list value =
  if atom value then []
  else car value :: retrieve_list (cdr value)

let opp = function
  | Down -> Up
  | Left -> Right
  | Up -> Down
  | Right -> Left
