open Ghc

let int i = GccTypes.TAG_INT i
let cons (a, b) = GccTypes.TAG_CONS (a,b)

let rec list = function
  | [] -> int 0
  | h :: t -> cons (h, list t)

let larg = function
  | Set_reg r        -> cons (int 0, int r)
  | Set_mem_at_reg r -> cons (int 1, int r)
  | Set_mem_at_cst c -> cons (int 3, int c)

let rarg = function
  | Pc           -> cons (int 0, int 8)
  | Reg r        -> cons (int 0, int r)
  | Mem_at_reg r -> cons (int 1, int r)
  | Cst c        -> cons (int 2, int c)
  | Mem_at_cst c -> cons (int 3, int c)

let arith = function
  | Add -> 3
  | Sub -> 4
  | Mul -> 5
  | Div -> 6
  | And -> 7
  | Or  -> 8
  | Xor -> 9

let cmp = function
  | Lt -> 10
  | Eq -> 11
  | Gt -> 12

let instr' = function
  | Set (l,r)             -> 0, [larg l; rarg r] (* MOV *)
  | Jmp r                 -> 0, [rarg Pc; rarg r] (* MOV *)
  | Arith (op, l, r)      -> arith op, [larg l; rarg r]
  | Cmp (op, addr, r1, r2) -> cmp op, [int addr; rarg r1; rarg r2]
  | Interrupt i           -> 13, [int i]
  | Halt                  -> 14, []
  | Undefined             -> raise Exit

let instr ins =
  let i, l = instr' ins in
  cons (int i, list l)

let ghc_value g = list (List.map instr (Array.to_list g.code))
